/*
 * Created by IntelliJ IDEA.
 * User: Mike
 * Date: Feb 19, 2004
 * Time: 3:10:46 PM
 */
package com.atlassian.confluence.rpc.soap.services;

import com.atlassian.confluence.rpc.NotPermittedException;
import com.atlassian.confluence.rpc.RemoteException;
import com.atlassian.confluence.rpc.soap.beans.RemoteUser;
import com.atlassian.confluence.rpc.soap.beans.RemoteUserInformation;
import com.atlassian.confluence.security.Permission;
import com.atlassian.confluence.security.PermissionManager;
import com.atlassian.confluence.security.SpacePermissionManager;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.user.PersonalInformation;
import com.atlassian.confluence.user.UserAccessor;
import com.atlassian.confluence.user.PersonalInformationManager;
import com.atlassian.confluence.util.GeneralUtil;
import com.atlassian.user.Group;
import com.atlassian.user.User;
import com.atlassian.user.EntityException;
import com.atlassian.user.search.page.Pager;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

public class UsersSoapService
{
    UserAccessor userAccessor;
    SpacePermissionManager spacePermissionManager;
    PermissionManager permissionManager;
    SoapServiceHelper soapServiceHelper;
    SpaceManager spaceManager;
    PersonalInformationManager personalInformationManager;

    public void setUserAccessor(UserAccessor userAccessor)
    {
        this.userAccessor = userAccessor;
    }

    public void setSpacePermissionManager(SpacePermissionManager spacePermissionManager)
    {
        this.spacePermissionManager = spacePermissionManager;
    }

    public void setSoapServiceHelper(SoapServiceHelper soapServiceHelper)
    {
        this.soapServiceHelper = soapServiceHelper;
    }

    public void setSpaceManager(SpaceManager spaceManager)
    {
        this.spaceManager = spaceManager;
    }

    public void setPermissionManager(PermissionManager permissionManager)
    {
        this.permissionManager = permissionManager;
    }

    private boolean canUserAdminister(User currentUser)
    {
        return (GeneralUtil.isSuperUser(currentUser) || permissionManager.hasPermission(currentUser, Permission.ADMINISTER, null));
    }

    public RemoteUser getUser(String username) throws RemoteException
    {
        return new RemoteUser(retrieveUser(username));
    }

    private User retrieveUser(String username)
            throws RemoteException
    {
        User user = userAccessor.getUser(username);
        if (user == null)
            throw new RemoteException("No user with that username found.");

        return user;
    }

    public boolean addUser(RemoteUser user, String password) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        userAccessor.addUser(user.getName(), password, user.getEmail(), user.getFullname(), new String[]{UserAccessor.GROUP_CONFLUENCE_USERS});
        return true;
    }

    /**
     * Delete the given user from the Confluence System.
     *
     * @param username is the name of the user that is to be deleted from the CONFLUENCE SYSTEM.
     * @throws RemoteException if user with the given name does not exist or if the authenticated user does not have
     *                         ADMINISTRATE CONFLUENCE PERMISSION or if THE USER HAS AUTHORED CONTENT
     */

    public boolean removeUser(String username) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        User user = this.retrieveUser(username);
        //check if user has authored content
        List userOwnedSpaces = spaceManager.getAuthoredSpacesByUser(username);
        List userEditedSpaces = spaceManager.getSpacesContainingPagesEditedBy(username);
        List userCommentedSpaces = spaceManager.getSpacesContainingCommentsBy(username);
        //if user has authored content then he cannot be deleted therefore, throw exception.
        if (!(userOwnedSpaces.isEmpty() && userEditedSpaces.isEmpty() && userCommentedSpaces.isEmpty()))
        {
            throw new RemoteException("cannot remove user because he has authored content");
        }
        userAccessor.removeUser(user);
        return true;
    }

    public boolean editUser(RemoteUser remoteUser) throws NotPermittedException, RemoteException
    {
        User currentUser = AuthenticatedUserThreadLocal.getUser();
        User givenUser = retrieveUser(remoteUser.getName());
        boolean updateRequired = false;

        // If the current user can't administrate
        if (!permissionManager.hasPermission(currentUser, Permission.EDIT, givenUser))
            throw new NotPermittedException("You are not logged in as the correct user, or you do not have the correct permissions to perform this action.");

        // Compare the user attributes
        if (!givenUser.getName().equals(remoteUser.getName()))
            throw new RemoteException("Cannot change the username");

        // Full name
        if (!givenUser.getFullName().equals(remoteUser.getFullname()))
        {
            updateRequired = true;

            givenUser.setFullName(remoteUser.getFullname());
        }

        // Email
        if (!givenUser.getEmail().equals(remoteUser.getEmail()))
        {
            updateRequired = true;

            givenUser.setEmail(remoteUser.getEmail());
        }

        if (updateRequired)
            userAccessor.saveUser(givenUser);

        return true;
    }

    public String[] getUserGroups(String username) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        User user = retrieveUser(username);
        Pager groups = userAccessor.getGroups(user);
        ArrayList groupsOfUser = new ArrayList();

        Iterator iter = groups.iterator();
        while (iter.hasNext())
        {
            Group group = (Group) iter.next();
            groupsOfUser.add(group.getName());
        }

        return (String[]) groupsOfUser.toArray(new String[1]);
    }

    public boolean addUserToGroup(String username, String groupname) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        User user = retrieveUser(username);
        Group group = userAccessor.getGroup(groupname);

        if (group == null)
        {
            throw new RemoteException("The group specified does not exist.");
        }
        userAccessor.addMembership(group, user);
        return true;
    }

    /**
     * Delete the user with name username from the group with name groupname.
     *
     * @param username  is the name of the user that is to be deleted from the group with wih name groupname.
     * @param groupname is the name of the group from which the user with name username is to be deleted.
     * @throws RemoteException if user/group with the given names does not exist or if the authenticated user does not have
     *                         ADMINISTRATE CONFLUENCE PERMISSION
     */
    public boolean removeUserFromGroup(String username, String groupname) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        User user = this.retrieveUser(username);
        Group group = userAccessor.getGroup(groupname);
        if (group == null)
        {
            throw new RemoteException("The group specified does not exist.");
        }

        userAccessor.removeMembership(group, user);
        return true;
    }


    public boolean addGroup(String groupname) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();

        if (userAccessor.getGroup(groupname) == null)
        {
            userAccessor.addGroup(groupname);
            return true;
        }

        return false;
    }

    /**
     * Delete all permissions (global and space level) permissions for the group with the given name.
     *
     * @param groupname is the group for which all the permissions has to be deleted
     * @throws RemoteException if group with the given name does not exist or if the authenticated user does not have
     *                         ADMINISTRATE CONFLUENCE PERMISSION
     */
    public boolean removeAllPermissionsForGroup(String groupname) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        Group group = userAccessor.getGroup(groupname);
        if (group == null)
        {
            throw new RemoteException("cannot delete permissions for non existing group");
        }
        spacePermissionManager.removeAllPermissionsForGroup(groupname);
        return true;
    }

    /**
     * Delete group with name groupname. If defaultGroupName is specified i.e not null and valid, then, add users
     * from the group with name groupname to the defaultgroup and deleteGroup
     *
     * @param groupname        is the group that is to be deleted
     * @param defaultGroupName is the default group to which the users from the group to be deleted will be added
     * @throws RemoteException if groups with givenname/defaultName does not exist or if  the authenticated user does not have
     *                         ADMINISTRATE CONFLUENCE PERMISSION
     */
    public boolean removeGroup(String groupname, String defaultGroupName) throws RemoteException
    {
        //Check if the authenticated user has ADMINISTRATE CONFLUENCE PERMISSION
        soapServiceHelper.assertCanAdministrate();

        //Retieve the group to be deleted
        Group group = userAccessor.getGroup(groupname);
        if (group == null)
        {
            throw new RemoteException("cannot remove non existing group");
        }

        //Check if user has specified a default group.
        boolean hasDefault = false;
        Group defaultGroup = null;
        if (defaultGroupName != null && defaultGroupName.length() != 0)
        {
            defaultGroup = userAccessor.getGroup(defaultGroupName);
            if (defaultGroup == null)
            {
                throw new RemoteException("default group does not exist");
            }
            hasDefault = true;
        }

        //If Default group is specified and present then add users to default before deleting group
        for (Iterator iterator = userAccessor.getMemberNames(group).iterator(); iterator.hasNext();)
        {
            String username = (String) iterator.next();
            if (hasDefault)
            {
                userAccessor.addMembership(defaultGroup.getName(), username);
            }
//            userAccessor.removeMembership(group.getName(), username);
        }

        //Perform safe remove now
        //now we remove all users, one by one. This will have to be improved

        userAccessor.removeGroup(group);
        spacePermissionManager.removeAllPermissionsForGroup(group.getName());

        return true;
    }

    public String[] getGroups() throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();
        Pager groups = userAccessor.getGroups();

        ArrayList groupsOfUser = new ArrayList();

        Iterator iter = groups.iterator();

        int i = 0;
        while (iter.hasNext())
        {
            Group group = (Group) iter.next();
            groupsOfUser.add(group.getName());
        }

        return (String[]) groupsOfUser.toArray(new String[1]);
    }

    public boolean deactivateUser(String username) throws NotPermittedException, RemoteException
    {
        soapServiceHelper.assertCanAdministrate();

        User user = retrieveUser(username);

        // If the user isn't active
        if (userAccessor.isDeactivated(user))
            throw new RemoteException("User has already been deactivated");

        userAccessor.deactivateUser(user);

        return true;
    }

    public boolean reactivateUser(String username) throws NotPermittedException, RemoteException
    {
        soapServiceHelper.assertCanAdministrate();

        User user = retrieveUser(username);

        // If the user hasn't already been deactivated
        if (!userAccessor.isDeactivated(user))
            throw new RemoteException("User is already active");

        userAccessor.reactivateUser(user);

        return true;
    }

    public String[] getActiveUsers(boolean viewAll) throws RemoteException
    {
        soapServiceHelper.assertCanAdministrate();

        Pager tempPager = null;
        List userList = new ArrayList();

        // Grab whatever users we need
        if (viewAll)
            tempPager = userAccessor.getUsers();
        else
            tempPager = userAccessor.getUsersWithConfluenceAccess();

        // Iterate through and covert to Usernames
        Iterator iter = tempPager.iterator();

        while (iter.hasNext())
        {
            User user = (User) iter.next();
            userList.add(user.getName());
        }

        return (String[]) userList.toArray(new String[userList.size()]);
    }

    public boolean changeMyPassword(String oldPass, String newPass) throws NotPermittedException, RemoteException
    {
        if (newPass == null || newPass.length() == 0)
            throw new RemoteException("New password cannot be null or empty");

        User currentUser = AuthenticatedUserThreadLocal.getUser();

        if (!userAccessor.authenticate(currentUser.getName(), oldPass))
            throw new NotPermittedException("The current password was incorrect. Please try again.");

        // Change the password and save the User
        try
        {
            userAccessor.alterPassword(currentUser, newPass);
        }
        catch (EntityException e)
        {
            throw new RemoteException(e);
        }

        return true;
    }

    public boolean changeUserPassword(String username, String newPass) throws NotPermittedException, RemoteException
    {
        // Check that the new and confirmation passwords are legit, then check that they are the same
        if (newPass == null || newPass.length() == 0)
            throw new RemoteException("New password cannot be null or empty.");

        User currentUser = AuthenticatedUserThreadLocal.getUser();
        User givenUser = retrieveUser(username);

        // If the current user can't administrate
        if (!canUserAdminister(currentUser))
            throw new NotPermittedException("You do not have the correct permissions to perform this action.");

        // Change the password and save the User
        givenUser.setPassword(newPass);
        userAccessor.saveUser(givenUser);

        return true;
    }

    public boolean setUserInformation(RemoteUserInformation userInfo) throws RemoteException
    {
        User currentUser = AuthenticatedUserThreadLocal.getUser();
        User givenUser = retrieveUser(userInfo.getUsername());

        // If the current user is not the given user, or they can't administrate
        if (!permissionManager.hasPermission(currentUser, Permission.EDIT, givenUser))
            throw new NotPermittedException("You are not logged in as the correct user, or you do not have the correct permissions to perform this action.");

        // Check values passed in
        if (userInfo.getContent() != null)
        {
            // We don't want to modify anything except the content, so copy the current personal information
            PersonalInformation newInfo = personalInformationManager.getPersonalInformation(givenUser.getName());
            PersonalInformation oldInfo = null;

            // If we didn't get anything back, create a new PersonalInformation instance for the user
            if (newInfo == null)
            {
                newInfo = new PersonalInformation(givenUser, userAccessor);
            }
            else
            {
                // Otherwise clone the current instance, so we have something to compare to
                try
                {
                    oldInfo = (PersonalInformation) newInfo.clone();
                }
                catch (CloneNotSupportedException e)
                {
                    throw new RemoteException("Error saving personal information: Clone not supported");
                }
            }

            if (newInfo.getId() != userInfo.getId())
                throw new RemoteException("Error saving personal information: ID cannot be changed");

            // If the values are the same, do nothing and exit
            if (newInfo.getContent().equals(userInfo.getContent()))
                return true;

            newInfo.setContent(userInfo.getContent());

            personalInformationManager.savePersonalInformation(newInfo, oldInfo);
        }

        return true;
    }

    public RemoteUserInformation getUserInformation(String username) throws RemoteException
    {
        User givenUser = retrieveUser(username);

        PersonalInformation info = personalInformationManager.getPersonalInformation(givenUser.getName());

        return new RemoteUserInformation(info);
    }

    public boolean hasUser(String token, String username)
    {
        User user = userAccessor.getUser(username);

        // If the user exists, return true
        return (user != null);
    }

    public boolean hasGroup(String groupname)
    {
        Group group = userAccessor.getGroup(groupname);

        // If the group exists, return true
        return (group != null);
    }

    public PersonalInformationManager getPersonalInformationManager()
    {
        return personalInformationManager;
    }

    public void setPersonalInformationManager(PersonalInformationManager personalInformationManager)
    {
        this.personalInformationManager = personalInformationManager;
    }
}